<?php

require(__DIR__ . '/lib/lib_xml.php');
use function WMSDFCL\lib\lib_xml\xml_decode;
use function WMSDFCL\lib\lib_xml\xml_find_content;
use function WMSDFCL\lib\lib_xml\xml_find_contents;
require(__DIR__ . '/lib/lib_picture.php');

class pictureXMLPlugin extends PluginBase{
	function __construct(){
		parent::__construct();
	}
	public function regiest(){
		$this->hookRegiest(array(
			'user.commonJs.insert'	=> 'pictureXMLPlugin.echoJs'
		));
	}
	public function echoJs($st,$act){
		if($this->isFileExtence($st,$act)){
			$this->echoFile('static/main.js');
		}
	}
	private function determine_tag_color($type) {
		$arr = [
			'red' => [255,20,20],
			'green' => [0,220,0],
			'blue'=> [40,64,255],
			'yellow' => [240,150,0],
			'unlimited' => [63,63,63],
			'limited' => [70,127,255],
			'crossing' => [160,160,160]
		];
		foreach($arr as $k =>$v) {
			if(strpos($type,$k) !== false) {
				return $v;
			}
		}
		return [255,255,255];
	}
	private function showError($content) {
		?>

<!DOCTYPE html>
<html lang="en">
<head>
	<meta charset="UTF-8">
	<meta http-equiv="X-UA-Compatible" content="IE=edge">
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<title><?php echo LNG('pictureXML.error') ?></title>
	<style>
		.error {
			margin: 16px;
			padding: 8px;
			background-color: #FFCDD2;
			border-radius: 3px;
			border: 1px solid #C62828;
			color: #C62828;
			font-size: 15px;
		}
	</style>
</head>
<body>
	<div class="error">
		<?php echo htmlspecial($content) ?>
	</div>
</body>
</html>

		<?php
		exit;
	}
	public function showPicture(){
		$error_level = error_reporting();
		error_reporting(0);

		$xml_path = _DIR($this->in['path']);
		if(!file_exists($xml_path)) {
			$this->showError(LNG('pictureXML.error.tan90'));
		} 
		$data = xml_decode(file_get_contents($xml_path));
		if(!is_array($data)) {
			$this->showError(LNG('pictureXML.error.malformed'));
		}

		$annotation_class = xml_find_content($data,'annotation');
		if($annotation_class === null) {
			$this->showError(LNG('pictureXML.error.xmlwrong'));
		}
		$_size = xml_find_content($annotation_class,'size');
		$width = intval(xml_find_content($_size,'width')[0]);
		$height = intval(xml_find_content($_size,'height')[0]);

		$pic_name = xml_find_content($annotation_class,'filename')[0];
		$pic_path = dirname($xml_path) . '/' . $pic_name;
		if(strpos($pic_name, '..') !== false || !file_exists($pic_path)) {
			// var_dump($pic_path);
			$this->showError(LNG('pictureXML.error.pic_tan90'));
		}

		$ext = get_path_ext($pic_path);
		$create_func = [
			'jpg' => 'imageCreateFromJPEG',
			'png' => 'imageCreateFromPNG',
			'bmp' => 'imageCreateFromBMP',
		][$ext];

		if(!$create_func) {
			$this->showError(LNG('pictureXML.error.pic_exterror'));
		}

		$img = $create_func($pic_path);
		imageSaveAlpha($img,true);
		imageAlphaBlending($img,true);
		$sizeX = imagesx($img);
		$sizeY = imagesy($img);
		if(!$img) {
			$this->showError(LNG('pictureXML.error.pic_malformed'));
		}

		
		$objects = xml_find_contents($annotation_class,'object');
		
		$text = basename($xml_path) . ' > ' . $pic_name . ' ('.$sizeX.'x'.$sizeY.'), ' . count($objects) . ' annotation' . (count($objects) > 1 ? 's' : '');
		$text_width = picture_text_width($text,'consolas',14);
		picture_rect($img,[8,8,8,44],8,$sizeY-14-24,min($sizeX-8,28+$text_width),$sizeY-8);
		picture_text(
			$img,
			$text,'consolas',
			14,0,[255,255,255],
			16,$sizeY-2-16
		);

		foreach($objects as $object) {
			$type = xml_find_content($object,'name')[0];
			$bndbox = xml_find_content($object,'bndbox');
			$xmin = intval(xml_find_content($bndbox,'xmin')[0]) * $sizeX / $width;
			$xmax = intval(xml_find_content($bndbox,'xmax')[0]) * $sizeX / $width;
			$ymin = intval(xml_find_content($bndbox,'ymin')[0]) * $sizeY / $height;
			$ymax = intval(xml_find_content($bndbox,'ymax')[0]) * $sizeY / $height;
			$tag_color = $this->determine_tag_color($type);
			$decorate_color = ($tag_color[0] * 0.3 + $tag_color[1] * 0.5 + $tag_color[2] * 0.2 >= 90) ? [0,0,0,44] : [255,255,255,44];
			picture_rect_o($img,$decorate_color,1,$xmin,$ymin,$xmax,$ymax);
			picture_rect_o($img,$tag_color,2,$xmin+1,$ymin+1,$xmax-1,$ymax-1);
			picture_rect($img,[0,0,0,63],$xmin+3,$ymin+3,$xmax-3,$ymax-3);
			$text_width = picture_text_width($type,'consolas-bold',14);
			picture_rect($img,$decorate_color,$xmin-2,$ymin-2-22,$xmin+$text_width+5,$ymin-2);
			picture_text(
				$img,
				$type,'consolas-bold',
				14,0,$tag_color,
				$xmin,$ymin-8
			);
		}

		header('Content-Type: image/png');
		imagepng($img);

		error_reporting($error_level);
	}
	public function index() {
		?>

<!DOCTYPE html>
<html lang="en">
<head>
	<meta charset="UTF-8">
	<meta http-equiv="X-UA-Compatible" content="IE=edge">
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<script src="static/js/lib/jquery-1.8.0.min.js?ver=448a"></script>
	<title><?php echo htmlspecial(LNG('pictureXML.window.title') . basename($this->in['path'])) ?></title>
	<style>
		body {
			background-color: #000;
			overflow: hidden;
			position:relative;
		}
		img {
			position: fixed;
			left: 50%;
			top: 50%;
			transform: translateX(-50%) translateY(-50%);
		}
	</style>
</head>
<body>
	<a href="?pluginApp/to/pictureXML/showPicture&path=<?php echo htmlspecial(urlencode($this->in['path'])) ?>" target="_blank"><img ondragstart="return false" class="img" src="?pluginApp/to/pictureXML/showPicture&path=<?php echo htmlspecial(urlencode($this->in['path'])) ?>"></a>
	<script>
		var ratio = 0;
		$('.img').on('load',() => {
			ratio = $(".img").width() / $('.img').height();
			function set_size() {
				var max_height = $(window).height() - 16;
				var max_width = $(window).width() - 16;
				var img_height = Math.min(max_height,max_width / ratio) - 0.1;
				$('.img').css('height',img_height + 'px');
			}
			set_size();
			$(window).on('resize',set_size);
		});
	</script>
</body>
</html>

		<?php
	}
}
