<?php
namespace WMSDFCL\lib\lib_xml;

function xml_decode($cont) {
	$parser = xml_parser_create('UTF-8');

	global $node_stack,$result,$char_store,$stack_top;
	$node_stack = [];
	$result = [];
	$char_store = '';
	$stack_top = 0;
	$node_stack[] = &$result;

	function handler_open($parser,$name,$attr) {
		global $node_stack,$result,$char_store,$stack_top;
		flush_str();
		$content_arr = [];
		$attr2 = [];
		foreach($attr as $k => $v) {
			$attr2[strtolower($k)] = $v;
		}
		$new_node = [
			'name' => strtolower($name),
			'attr' => $attr2,
			'content' => &$content_arr
		];
		$node_stack[$stack_top][] = &$new_node;
		$stack_top++;
		$node_stack[$stack_top] = &$content_arr;
	}

	function handler_close($parser,$name) {
		global $node_stack,$result,$char_store,$stack_top;
		flush_str();
		$stack_top--;
	}

	function handler_char($parser,$char) {
		global $node_stack,$result,$char_store,$stack_top;
		$char_store .= $char;
	}

	function flush_str() {
		global $node_stack,$result,$char_store,$stack_top;
		$char_store = trim($char_store);
		if($char_store != ''){
			$node_stack[$stack_top][] = $char_store;
		}
		$char_store = '';
	}

	xml_set_character_data_handler($parser,'WMSDFCL\lib\lib_xml\handler_char');
	xml_set_element_handler($parser,'WMSDFCL\lib\lib_xml\handler_open','WMSDFCL\lib\lib_xml\handler_close');
	xml_parse($parser,$cont,true);
	flush_str();

	return $result;
}

function xml_specials($str) {
	return str_replace(['&','<','>','"'],['&amp;','&lt;','&gt;','&quot;'],$str);
}

function xml_indent($pretty_print,$n) {
	$ans = '';
	if($pretty_print) {
		for($i=0;$i<$n;$i++) {
			$ans .= "    ";
		}
	}
	return $ans;
}

function xml_enter($pretty_print) {
	$ans = '';
	if($pretty_print) $ans .= "\n";
	return $ans;
}

function xml_should_inline(&$content) {
	if(count($content) == 0) return true;
	if(count($content) >= 2) return false;
	if(!is_array($content[0])) {
		return true;
	}
	return false;
}

function xml_encode($arr,$pretty_print=false,$level=0) {
	if(!is_array($arr)) {
		$ans = xml_indent($pretty_print,$level) . xml_specials($arr) . xml_enter($pretty_print);
		return $ans;
	}
	if(is_array($arr) && !isset($arr['name'])) {
		$ans = '';
		foreach($arr as $val) {
			$ans .= xml_encode($val,$pretty_print,$level);
		}
		return $ans;
	}
	$ans = '';
	$should_inline = xml_should_inline($arr['content']);
	$ans .= xml_indent($pretty_print,$level);
	$ans .= '<';
	$ans .= $arr['name'];
	foreach($arr['attr'] as $k => $v) {
		$ans .= ' ';
		$ans .= $k;
		$ans .= '"' . xml_specials($v) . '"';
	}
	$ans .= '>';
	if(!$should_inline) $ans .= xml_enter($pretty_print);
	$ans .= xml_encode($arr['content'],$pretty_print && (!$should_inline),$level+1);
	if(!$should_inline) $ans .= xml_indent($pretty_print,$level);
	$ans .= "</" . $arr['name'] . '>' . "\n";
	return $ans;
}

function &xml_find_content($arr,$name) {
	foreach($arr as &$v) {
		if($v['name'] == $name) {
			return $v['content'];
		}
	}
	return null;
}

function &xml_find_contents($arr,$name) {
	$ret = [];
	foreach($arr as &$v) {
		if($v['name'] == $name) {
			$ret[] = $v['content'];
		}
	}
	return $ret;
}
